.BG
.FN yags
.TL
Yet Another GEE Solver (CUTYAGS $Revision: 2.14 $)
.DN
solve generalized estimating equation
.CS
yags(formula, id, weights=NULL, cor.met=NULL, family=gaussian, 
alpfun=NULL, scalefun=LZ.scalefun, wcorigen=identni, tol=1e-05, 
contrasts=NULL, corstr=c("independence", "exchangeable", 
"ar1", "unstructured"), maxiter=25, alphainit=0, hardcode=F, 
binit=NULL, verbose=F, Ua.tol=1e-06, Ua.maxit=20, Ua.gridlo=0.1, 
Ua.gridhi=0.9, Ua.gridnpts=10, Ua.secantdel=0.01)
.RA
.AG formula
standard glm formula
.AG id
vector of cluster discriminators
.OA
.AG weights
vector of glm weights
.AG cor.met
dependence coordinates; can be matrix
.AG family
glm family
.AG alpfun
Specially constructed splus function that
returns an estimate of alpha, the working
correlation parameter; see the noweb doc for details.
Ignored with warning when hardcode==T.
.AG scalefun
specially constructed splus function
that returns an estimate of the scale
parameter.
Ignored when hardcode==T, the Liang-Zeger 1986
proposal estimator is used.
.AG wcorigen
specially constructed splus function
that returns the inverse of the working
correlation matrix.  Ignored for hardcode == T.
.AG tol
convergence criterion for regression parameters
.AG contrasts
glm contrasts
.AG corstr
tag describing the working correlation model,
currently limited to c("independence", "ar1",
"exchangeable", "unstructured") for hardcode==T,
but not necessary when alpfun and wcorigen are
supplied for a pure Splus5 run
.AG maxiter
maximum number of iterations for beta
.AG alphainit
initial value of working correlation parameter;
unnecessary when closed-form estimators of alpha
.AG hardcode
logical indicator of whether to use C++ engine; if
true, only a few working correlation structure
options are available without direct modification
of C++ code
.AG binit
initial value of regression parameter.  typically
unnecessary.
.AG verbose
logical indicator of whether realtime iteration information
is to be printed
.AG Ua.tol
convergence criterion for supplemental estimating equation
.AG Ua.maxit
iteration limit for supplemental estimating equation
.AG Ua.gridlo
lower bound for supplemental estimating equation initialization
grid
.AG Ua.gridhi
upper bound for supplemental estimating equation initialization
grid
.AG Ua.gridnpts
number of grid points for supplemental estimating equation initialization
.AG Ua.secantdel
offset for secant method for first supplemental estimating equation iteration
.RT
an object of class yags, with components
coefficients naive.parmvar robust.parmvar alpha
phi linear.predictors fitted.values residuals
iter family rank errorcode sealp call hardcode corstr
.SE
for major versions > 3, dist is overwritten
as it fails on scalar argument
.DT
The hardcode option selects a C++ implementation that
emulates the Splus implementation of yags and solves
GEEs much more rapidly.  However, the full flexibility
of yags is not yet available when hardcode==T.

There are various approaches to estimating working
correlation model components of GEE, and in this
version of yags, hardcode and pure Splus5 solutions
are not designed to agree exactly.  In particular,
the AR(1) working model in an hardcode==F run will
proceed by using prop.ar1alp() (a closed form
estimator provided in SAS (tm) proc genmod.
The
AR(1) working model with hardcode==T solves instead
a supplementary estimating equation using
the secant method (e.g., Thisted 1988) initialized
on a grid of values (see the Ua.* parameters
of yags).  These two approaches to fitting the
same model (using
different sets of estimating equations) will typically
yield slightly different numerical results.

With hardcode==F, selection of alpfun=UG.alpfun.fom
(fom denoting "First Order Markovian") will give
a pure Splus5 implementation of the same set of estimating equations
as is solved under hardcode==T, but the solution procedure
involves ms() and therefore need not lead to identical
convergent results.

The next version will include a cleaner approach
to specification of the target equations to be solved.

.SH REFERENCES
Liang and Zeger Biometrika 1986
.SA
glm
.EX
NN1.slow <- yags(formula = BAlr.y ~ Alr.trt, id = Test.id, family = binomial, alpfun =
        LZ.exchalp, scalefun = LZ.scalefun, wcorigen = excoriput)

NN1.fast <- yags(formula = BAlr.y ~ Alr.trt, id = Test.id, family = binomial,
        corstr="exchangeable", hardcode=T)

.KW GLMs, longitudinal data
.WR

